package com.loafle.overflow.central.module.sensor.service;

import com.loafle.overflow.central.module.sensor.dao.SensorItemDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.meta.MetaDisplayItemMapping;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;

import com.loafle.overflow.service.central.sensor.SensorItemService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorItemService")
public class CentralSensorItemService implements SensorItemService {

  @Autowired
  private SensorItemDAO sensorItemDAO;
  @Autowired
  private CentralSensorService sensorService;

  @Transactional
  public SensorItem regist(Sensor sensor, MetaDisplayItemMapping metaDisplayItemMapping) throws OverflowException {
    SensorItem sensorItem = new SensorItem();
    sensorItem.setMetaDisplayItemMapping(metaDisplayItemMapping);
    sensorItem.setSensor(sensor);
    SensorItem retSensorItem = this.sensorItemDAO.save(sensorItem);
    if (null != retSensorItem) {
      this.sensorService.increaseSensorItemCount(sensor.getId());
    }
    return retSensorItem;
  }

  @Override
  public SensorItem read(Long id) throws OverflowException {
    return this.sensorItemDAO.findById(Long.valueOf(id)).get();
  }

  @Override
  public List<SensorItem> readAllBySensorID(Long sensorID) throws OverflowException {
    return this.sensorItemDAO.findAllBySensorId(sensorID);
  }

  @Transactional
  @Override
  public void remove(Long sensorItemID) throws OverflowException {
    SensorItem sensorItem = this.read(sensorItemID);
    if (null == sensorItem) {
      throw new OverflowException(String.format("ID[%d] of SensorItem is not valid", sensorItemID));
    }
    Sensor sensor = sensorItem.getSensor();
    if (null == sensor) {
      throw new OverflowException(String.format("ID[%d] of Sensor is not valid", sensorItem.getSensor().getId()));
    }
    sensor.setItemCount(sensor.getItemCount() - 1);
    this.sensorService.regist(sensor);
    this.sensorItemDAO.deleteById(sensorItemID);
  }

  public void removeAllBySensorID(Long sensorID) throws OverflowException {
    this.sensorItemDAO.deleteAll(this.readAllBySensorID(sensorID));
  }

}
